

package app.navigator.gui;

import app.navigator.util.*;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;


public class ChartComp extends JPanel implements MouseListener{

  final int redCircle=20;  // The radius of the circle in the position mark
  final int greenCross=20; // The size of the cross in the position mark
  Image img=null;          // The chart being show on screen
  Image orgImg=null;       // A previous chart
  boolean valid=false;     // True if position info has timed out
  Point pred=null;         // Predicted position
  Point pos=null;          // Position of the mark
  int precision;           // The estimated precision on the chart in pixels
  int scale;               // The number of pixels per 100 (or 50) meters.
  boolean trace;           // If true we plot a trace line.
  int thickness;           // How thick the red circle shall be
  Point[] tracePoints=new Point[2000];  // To remember where we have plotted a blue line.
  int traceIndex;          // Where in the array above we shall store next point.
  Point lastPos=null;      // Used so we don't save same point twice.
  NavFrame parent=null;       // The frame in which this component was created.
  final int BORDER=5;      // It seems some extra margin is needed otherwise image is cut */
  Point mousePos=null;     // Position for mouse click
  Object objectToNotify=null;
  int heading = 180;

	double zoomFactor = 1.0;

	int range = 0;

	private ChartData chartdata; // current chart



	private HashMap table = new HashMap();
	private String id ="";

	/** A time for keeping track of double-clicking. */
	private Date doubleclick = new Date();

	private String current;

	private Point dest;

	private double dest_heading = -1;


  public ChartComp(NavFrame parent, boolean trace, int thickness)
  {

    this.parent=parent;

    loadChart("uo.xml");

    this.addMouseListener(this);
  }





  synchronized public Dimension getPreferredSize()
  {
    if (img!=null)
    {

      // Wait until the image is loaded so we know the size
      int i=0;
      while ((i<30) && (img.getWidth(this)<0))
      {
        try {this.wait(100);} catch (InterruptedException e) {;}
        i++;
      }



      return new Dimension(img.getWidth(this)+BORDER, img.getHeight(this)+BORDER);
    }
    else
    {
      return new Dimension(120,100);
    }
  }


  public void setNotify(Object objectToNotify)
  {
    this.objectToNotify=objectToNotify;
  }



  static void error(String s)
  {
    System.err.println("ChartComp: " + s);
  }



	public Point getLocation(){
		return pos;
	}



  synchronized void drawCross(Graphics g)
  {
    try
    {

		java.util.Collection all = table.keySet();
		Object[] objs = all.toArray();

		String aid ="";
		Point pos;
		int heading = 90;

		for (int i=0; i<objs.length; i++) {
			aid = (String)objs[i];
			pos = chartdata.transPos((Position)(table.get( objs[i])));

			heading = (int)(((Position)(table.get( objs[i]))).getHeading());

			//draw arrow to where user wants to go.
			if( dest_heading >= 0 && dest != null ){

  				g.setColor(Color.blue);

				int part1 = (int)(dest_heading/45);
				int rem1 = (int)(dest_heading%45);
				int deg1 = part1*45;
				if(rem1>=(45/2))
					deg1 += 45;
				if(deg1 == 360)
					deg1 = 0;

				int adj = (int)(Math.sin(Math.toRadians(deg1))*30);
				int opp = (int)(Math.cos(Math.toRadians(deg1))*30);

				int a1 = (int)(Math.sin(Math.toRadians(deg1+180-32))*14);
				int o1 = (int)(Math.cos(Math.toRadians(deg1+180-32))*14);

				int a2 = (int)(Math.sin(Math.toRadians(deg1+180+32))*14);
				int o2 = (int)(Math.cos(Math.toRadians(deg1+180+32))*14);


				g.drawLine( (int)(zoomFactor*(pos.x)),(int)(zoomFactor*(pos.y)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );
				g.drawLine( (int)(zoomFactor*(pos.x-1)),(int)(zoomFactor*(pos.y-1)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );
				g.drawLine( (int)(zoomFactor*(pos.x-1)),(int)(zoomFactor*(pos.y)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );
				g.drawLine( (int)(zoomFactor*(pos.x+1)),(int)(zoomFactor*(pos.y+1)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );
				g.drawLine( (int)(zoomFactor*(pos.x+1)),(int)(zoomFactor*(pos.y)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );
				g.drawLine( (int)(zoomFactor*(pos.x)),(int)(zoomFactor*(pos.y+1)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );
				g.drawLine( (int)(zoomFactor*(pos.x)),(int)(zoomFactor*(pos.y-1)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );
				g.drawLine( (int)(zoomFactor*(pos.x+1)),(int)(zoomFactor*(pos.y-1)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );
				g.drawLine( (int)(zoomFactor*(pos.x-1)),(int)(zoomFactor*(pos.y+1)), (int)(zoomFactor*(pos.x))+opp,(int)(zoomFactor*(pos.y))-adj );

				int[] xp = new int[3];
				xp[0] = (int)(zoomFactor*(pos.x))+opp;
				xp[1] = (int)(zoomFactor*(pos.x))+opp+o1;
				xp[2] = (int)(zoomFactor*(pos.x))+opp+o2;

				int[] yp = new int[3];
				yp[0] = (int)(zoomFactor*(pos.y))-adj;
				yp[1] = (int)(zoomFactor*(pos.y))-adj-a1;
				yp[2] = (int)(zoomFactor*(pos.y))-adj-a2;

				g.fillPolygon(xp,yp,3);

				g.fillOval((int)(zoomFactor*(dest.x-3)),(int)(zoomFactor*(dest.y-3)),(int)(zoomFactor*(7)),(int)(zoomFactor*(7)));

			}//

			if( aid.equals(id) )
				g.setColor(Color.red);
			else
				g.setColor(Color.green);

			g.setFont(new Font("Default", Font.BOLD, 14));
			g.fillOval((int)(zoomFactor*(pos.x-3)),(int)(zoomFactor*(pos.y-3)),(int)(zoomFactor*(7)),(int)(zoomFactor*(7)));
			//g.drawString(aid, (int)(zoomFactor*(pos.x+4)),(int)(zoomFactor*(pos.y+10)));

			int part = heading/45;
			int rem = heading%45;
			int deg = part*45;
			if(rem>=(45/2))
				deg += 45;
			if(deg == 360)
				deg = 0;

			int adj1 = (int)(Math.sin(Math.toRadians(deg-55))*35);
			int opp1 = (int)(Math.cos(Math.toRadians(deg-55))*35);

			int adj2 = (int)(Math.sin(Math.toRadians(deg+55))*35);
			int opp2 = (int)(Math.cos(Math.toRadians(deg+55))*35);

			g.drawLine( (int)(zoomFactor*(pos.x)),(int)(zoomFactor*(pos.y)), (int)(zoomFactor*(pos.x))+opp1,(int)(zoomFactor*(pos.y))-adj1 );
			g.drawLine( (int)(zoomFactor*(pos.x)),(int)(zoomFactor*(pos.y)), (int)(zoomFactor*(pos.x))+opp2,(int)(zoomFactor*(pos.y))-adj2 );


		}


    }catch(Exception e){}
  }



	///
	//
	// This draws a trace line!!
	//
	//
  public synchronized void plottPos(){

    Graphics g=getGraphics();

    drawCross(g);

    //if ((this.pos!=null) && (pos!=null) && trace)
    //{
    //  g.setPaintMode();
    //  g.setColor(Color.blue);
      //g.drawLine(pos.x,pos.y,this.pos.x,this.pos.y);
    //}

    //this.pos=pos;
    //this.pred=pred;
    valid=true;
    drawCross(g);



    g.dispose();
  }




  public synchronized void updatePos(HashMap table, String id)
  {
	this.table = table;
	this.id = id;


	  //int temp = (int)(nheading/45);
	  //this.heading = temp*45;
    plottPos();
    //if (trace) {tracePos(pos);}
  }






  	public synchronized void timeOut(){

    	Graphics g=getGraphics();
    	drawCross(g);
    	valid=false;
    	drawCross(g);
    	g.dispose();
  	}




  	public void update(Graphics g){

    	paint(g);
  	}



  	public synchronized void paint(){

    	Graphics g=getGraphics();
    	paint(g);
    	g.dispose();
  	}



  public synchronized void paint(Graphics g)
  {
    if (img!=null) {g.drawImage(img, 0, 0, this);}

    g.setPaintMode();
    g.setColor(Color.blue);

    //if (trace)
    {
      Point p=null;
      int j=traceIndex;
      for (int i=0;i<tracePoints.length;i++)
      {
        if (++j>=tracePoints.length) {j=0;}
        if (tracePoints[j]!=null)
        {
          if (p!=null)
          {
            //g.drawLine(p.x,p.y,tracePoints[j].x,tracePoints[j].y);
          }
        }
        p=tracePoints[j];
      }
    }

    drawCross(g);
    g.dispose();
  }




    /* ===================================================================== */
    /** This method checks for two kinds of mouse clicks.  If first checks for
     *  a right mouse click which is for rotating a user.(changing user's
     *  facing direction)
     *  This mehtod also checks for mouse left button double clicks.  Based on
     *  the location of click and the timing, it will lauch a PropDialog for
     *  the User that was double clicked on.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseClicked(MouseEvent e){

		int btn = e.getButton();
		Point p = e.getPoint();

		if( btn == 1 ){

			double dif = 100000;
			double temp;

			java.util.Collection all = table.keySet();
			Object[] objs = all.toArray();
			String aid ="";
			Point pos;

			for (int i=0; i<objs.length; i++) {
				aid = (String)objs[i];
				pos = chartdata.transPos((Position)(table.get( objs[i])));

				temp = diff(p, pos);

				if( temp < dif ){
					dif = temp;
					current = aid;
				}

			}


			if( dif < 18 ){


				// Check time to see if it is double-click. double-click if < .75 sec.
				if( ((new Date()).getTime() - doubleclick.getTime()) < 750 ){

					parent.selectAgent(current);
					System.out.println("double clicked .. .. . .");

				}else{
					//Get time of click for checking for double-click next time.
					doubleclick = new Date();
				}

			} //if

		} //if


	} //method: mouseClicked





    /* ===================================================================== */
    /** This method is used to get the offset position when we click and drag
     *  an agent on the map.  Offset is needed for correctly drawing the agent
     *  on the map.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mousePressed(MouseEvent e){
	} //method: mousePressed




    /* ===================================================================== */
    /** This method lets the object know that the user has released from a
     *  mouse drag.  It sends a location change event, so that the location
     *  of the current user can be sent to the iPAQ application.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseReleased(MouseEvent e){
	} //method: mouseReleased



    /* ===================================================================== */
    /** This method lets the object know that the user has released from a
     *  mouse drag.  It sends a location change event, so that the location
     *  of the current user can be sent to the iPAQ application.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseEntered(MouseEvent e){
	} //method: mouseReleased

    /* ===================================================================== */
    /** This method lets the object know that the user has released from a
     *  mouse drag.  It sends a location change event, so that the location
     *  of the current user can be sent to the iPAQ application.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseExited(MouseEvent e){
	} //method: mouseReleased



     /* ===================================================================== */
    /** This method returns the pixel distance between two points.
     *
     *  @param  p1   The first point.
     *  @param  p2   The second point.
     *
     */
	private double diff(Point p1, Point p2){

		double x = p1.getX() - p2.getX();
		if( x < 0 )
			x = x * -1;

		double y = p1.getY() - p2.getY();
		if( y < 0 )
			y = y * -1;


		return Math.sqrt( x*x + y*y );

	} //method: diff



    /* ===================================================================== */
    /** This method returns the pixel distance between two points.
     *
     *  @param  p1   The first point.
     *  @param  p2   The second point.
     *
     */
	private double getDegrees(Point p1, Point p2){

		double x = p1.getX() - p2.getX();
		double y = p1.getY() - p2.getY();


		double diff = Math.sqrt( x*x + y*y );

		return Math.toDegrees( Math.asin( (p1.getY()-p2.getY())/diff ) );

	} //method: diff


    /* ===================================================================== */
    /** This method returns the chartdata kept inside this object used for
     *  turning pixel coordinates into real GPS coordinates.
     *
     *  @return  ChartData   The chart data used in this object to return.
     *
     */
	public ChartData getChartData(){

		return chartdata;

	} //method: getChartData



	public void zoomOut(){

		zoomFactor = zoomFactor/1.5;

		int x = orgImg.getWidth(null);
		int y = orgImg.getHeight(null);

		img = orgImg.getScaledInstance((int)(x*zoomFactor), (int)(y*zoomFactor), Image.SCALE_SMOOTH);

		this.paint();

	}



	public void zoomIn(){

		zoomFactor = zoomFactor*1.5;

		int x = orgImg.getWidth(null);
		int y = orgImg.getHeight(null);

		img = orgImg.getScaledInstance((int)(x*zoomFactor), (int)(y*zoomFactor), Image.SCALE_SMOOTH);

		this.paint();

	}

	public double getZoomFactor(){
		return zoomFactor;
	}



    /* ===================================================================== */
    /** This method loads new chart data, by first reading the given xml file
     *  which contains map name and reference point information.  Then the new
     *  ChartData will give us the map image to use.
     *
     *  @param  name   file name for new data to use.
     *
     */
	public void loadChart(String name){

		System.out.println("ChartComp is loading the chart");

		this.setVisible(false);

		this.readData(name);

		if (img!=null) {img.flush();}

		img = getToolkit().getImage("files/app/gps/"+chartdata.imgFileName);
		orgImg = getToolkit().getImage("files/app/gps/"+chartdata.imgFileName);


		this.setSize(this.getPreferredSize());


		this.setVisible(true);
		repaint();

	} //method: loadChart



    /* ===================================================================== */
    /** This method loads new chart data, by first reading the given xml file
     *  which contains map name and reference point information.
     *
     *  @param  file_name   Name of XML file that contains new ChartData info.
     *
     */
 	public void readData(String file_name){

    	chartdata = null;

		InputParser ip = new InputParser(file_name);

		try{
			chartdata=ChartData.loadChartData(ip);
		}catch (IOException e) {}


  	} //method: readData



	public void setDest(double heading, Point d ){
		dest_heading = heading;
		dest = d;
		repaint();

	}

	public void resetDest(){
		dest = null;
		dest_heading = -1;
	}


}

