/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * adapted from: Henrik Bjorkman
 * @version 1.0
 * </pre>
 */

package app.navigator.util;

import java.io.*;
import java.awt.*;
import java.text.*;
import java.util.*;




public class ChartData{


  	public String imgFileName;
  	public ReferencePoint[] ref=new ReferencePoint[6];
  	public int nref=0;
  	public double north=0, south=0, east=0, west=0;
  	double A=0, B=0, C=0, D=0, E=0, F=0; //for the new calibration routine
  	public long diff=-1;
  	public String title=null;
  	public String datum;
  	public String compDatum;
  	public double compLat=0,compLng=0;



  	public int addRefPoint(ReferencePoint r){

    	if (nref>=ref.length) {return(1);}
    	ref[nref++]=r;
    	return(0);

  	}



  	public String toString(){

		String s=new String("begin ChartData\n");

		s+="name "+imgFileName+"\n";

		if (datum!=null) s+="datum "+datum+"\n";

		s+="lim "+ReferencePoint.toStringLatLng(south,'N','S')+" "+
		 ReferencePoint.toStringLatLng(north,'N','S')+" "+
		 ReferencePoint.toStringLatLng(west,'E','W')+" "+
		 ReferencePoint.toStringLatLng(east,'E','W')+"\n";

		for (int i=0; i<nref; i++)
		{
		  s+="ref "+ref[i].toString()+"\n";
		}

		if (compDatum!=null)
		{
		  s+="comp "+imgFileName+"\n";
		}

		if (title!=null) s+="title "+title+"\n";

		s+="end\n";

		return(s);
  	}




  	public boolean onChart(Position p){

    if ((north!=0) && (south!=0))
    	return((p.getLatitude()<=north) && (p.getLatitude()>=south) && (p.getLongitude()>=west) && (p.getLongitude()<=east));
    //if ((ref[0]==null)||(ref[1]==null)) error("No config data for "+imgFileName);
    return( (ref[0].lat>=p.getLatitude()) && (ref[1].lat<=p.getLatitude()) && (ref[0].lng<=p.getLongitude()) && (ref[1].lng>=p.getLongitude()) );

  	}



  	public int transPosX(double lat, double lng){

    	//return((new Double(A*lng+B*lat+C)).intValue());
    	return (int)Math.round((A*lng+B*lat+C));

  	}



  	public int transPosY(double lat, double lng){

    	//return((new Double(D*lng+E*lat+F)).intValue());
    	return (int)Math.round((D*lng+E*lat+F));

  	}



  	public double transPosLng(int x, int y){

    	return((E*x-B*y-(E*C-B*F))/(E*A-B*D));
  	}



  	public double transPosLat(int x, int y){

    	return((D*x-A*y-(D*C-A*F))/(D*B-A*E));
  	}




  	// Calculate the parameters to be used when translating GPS
  	// position to pixel coordinates on the chart.
  	public void findCalibrationParams(){

		ReferencePoint r0=ref[0];
		ReferencePoint r1=ref[1];
		ReferencePoint r2=ref[2];

		if (nref>=3)
		{
		  double tmp1=0, tmp2=0, tmp3=0, Tc=0, Nc=0, Tf=0, Nf=0;
		  tmp1=r0.lat*r2.lng-r2.lat*r0.lng;
		  tmp2=r1.lat*r0.lng-r0.lat*r1.lng;
		  tmp3=r0.lng-r1.lng;

		  Tc=(r0.lng*r1.x-r1.lng*r0.x)*tmp1+(r2.x*r0.lng-r0.x*r2.lng)*tmp2;
		  Nc=(r0.lng-r2.lng)*tmp2+tmp3*tmp1;
		  C=Tc/Nc;
		  B=(-C*tmp3+r0.lng*r1.x-r1.lng*r0.x)/tmp2;
		  A=(r0.x-C-B*r0.lat)/r0.lng;

		  Tf=(r0.lng*r1.y-r1.lng*r0.y)*tmp1+(r2.y*r0.lng-r0.y*r2.lng)*tmp2;
		  Nf=(r0.lng-r2.lng)*tmp2+tmp3*tmp1;
		  F=Tf/Nf;
		  E=(-F*tmp3+r0.lng*r1.y-r1.lng*r0.y)/tmp2;
		  D=(r0.y-F-E*r0.lat)/r0.lng;

		}
		else if (nref>=2)
		{
		  A=(r1.x-r0.x)/(r1.lng-r0.lng);
		  B=0;
		  C=-A*r0.lng+r0.x;
		  D=0;
		  E=(r1.y-r0.y)/(r1.lat-r0.lat);
		  F=-E*r0.lat+r0.y;
		}
		else
		{
		  //error("To few reference points");
		}
		//debug("A "+A+" B "+B+" C "+C+" D "+D+" E "+E+" F "+F);

		// If we have more than 3 ref points we use them to check the transposition.
		// This gives an estimated error in pixels.
		for (int i=3;i<nref;i++)
		{
		  int dx=transPosX(ref[i].lat,ref[i].lng)-ref[i].x;
		  int dy=transPosY(ref[i].lat,ref[i].lng)-ref[i].y;
		  long d2=Math.round(Math.sqrt(dx*dx+dy*dy));
		  if (d2>diff) {diff=d2;}
		}
  	}




  	// Translate a GPS position to pixel position on chart.
  	public Point transPos(Position pos){

		if (pos==null) return(null);
		Point p=new Point();
		p.x=transPosX(pos.getPureLat()+compLat, pos.getPureLng()+compLng);
		p.y=transPosY(pos.getPureLat()+compLat, pos.getPureLng()+compLng);
	   // debug("lat "+pos.lat+" lng "+pos.lng+" x "+p.x+" y "+p.y);
		return(p);
  	}



  	// Translate a pixel position to Lat/Lng position on chart.
  	public Position transPos(Point p){

		if (p==null) return(null);
		Position pos=new Position();
		pos.setLatitude(transPosLat(p.x, p.y)-compLat);
		pos.setLongitude(transPosLng(p.x, p.y)-compLng);

		//debug("reverse lat "+pos.lat+" lng "+pos.lng+" x "+p.x+" y "+p.y);
		return(pos);

  	}





	public static ChartData loadChartData(InputParser ip) throws IOException{

		ChartData d=new ChartData();


		d.imgFileName = ip.getMapFilename();
		System.out.println( "map file name: "+d.imgFileName);

		Vector refpoints = ip.getRefPoints();

		System.out.println( "We have "+refpoints.size()+" reference points." );
		for(int i=0; i <refpoints.size(); i++ ){

			if (d.nref>=d.ref.length) {
				throw new IOException("to much data");
			}
			d.ref[d.nref++]= ((ReferencePoint)refpoints.elementAt(i));

		}

		d.findCalibrationParams();

		return(d);

  	}


} //class: ChartData
