/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package transport;

import transport.protocol.*;

import java.io.*;
import java.net.*;
import java.util.*;


/* ====================================================================== */
/** This class
 *
 */
public class SocketReceiver extends Thread {

	private int port;

	//should change this to vector of listeners?
	//private SocketReceiverListener listener;


	/** Components that want to listen for context events. */
	private Vector listeners = new Vector();


    /* ===================================================================== */
    /** This constructor initializes the user.
     *
     *  @param  p   The port the ipaq gps receiver is listening on.
     *
     */
	public SocketReceiver(SocketReceiverListener listener, int port){

		this.port = port;
		this.addSocketReceiverListener(listener);

	}

    /* ===================================================================== */
    /** This constructor initializes the user.
     *
     *  @param  p   The port the ipaq gps receiver is listening on.
     *
     */
	public SocketReceiver(int port){

		this.port = port;

	}

    /* ===================================================================== */
    /** This
     *
     */
	public void run(){

		try{

			ServerSocket listen = new ServerSocket(port);

			while(true){

				try{

					//System.out.println ("SocketReceiver:  step 1 listening on port "+port);

					Socket s;

					s = listen.accept();


					//System.out.println ("SocketReceiver:  step 2 got a client");


					BufferedInputStream bufin = new BufferedInputStream(s.getInputStream());
					ObjectInputStream objin = new ObjectInputStream(bufin);

					//System.out.println( "SocketReceiver:  Receiving From: " + s.getInetAddress() + " " + s.getPort() + " (local address= " + s.getLocalAddress() + ", local port+ " + s.getLocalPort() + ")");

					Object msg = objin.readObject();
					objin.close();

					try{
						if( msg instanceof RangeMessage ){

							//this should be sent directly to a peer. (not through iSIM)
							//data inside the message inside the RangeMessage can be encrypted.
							SocketSender ss = new SocketSender((Message)(((RangeMessage)msg).getData()));
							ss.start();
						}
					}catch(Exception ee){}

					fireSocketReceiverEvent((Message)msg);



					//System.out.println( "SocketReceiver:  Received Object ");
				}catch(Exception ee){ee.printStackTrace();}

			}

		}catch(Exception e){System.out.println(e);}

	} //method: run




	/* ===================================================================== */
    /** This methods adds a listener to the list of listeners this
     *  ContextManager knows about.
     *
     *  @param  listener
     *
     */
    public void addSocketReceiverListener(SocketReceiverListener listener){

		listeners.add(listener);

    } //method: addContextListener




	/* ===================================================================== */
    /** This method removes a listener from the list of listeners that
     *  this ContextManager knows about.
     *
     *  @param  listener
     *
     */
    public void removeSocketReceiverListener(SocketReceiverListener listener){

		listeners.remove(listener);

    } //method: removeContextListener




	/* ===================================================================== */
    /** This method notifies all subscribed listeners of the Context event.
     *
     *  @param  ce
     *
     */
    public void fireSocketReceiverEvent(Message ce){

		SocketReceiverListener[] informMe = new SocketReceiverListener[listeners.size()];
		informMe = (SocketReceiverListener[])listeners.toArray(informMe);

		for(int i = 0; i < informMe.length; i++){

			informMe[i].incomingMessage(ce);
		}

    } //method: fireContextEvent




} //class: SocketReceiver