/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui.tray;

import gui.*;
import controller.*;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;



/* ====================================================================== */
/** This is the JDialog is designed to be a tool-tray for a the main frame
 *  in the GPS Simulator.  It contains various tabs for view information,
 *  adding/removing agents, and other tasks.
 *
 */
public class TrayDialog extends JDialog {

	/** The border layout to use for the Tray. */
    private BorderLayout bdrLayMain = new BorderLayout();

	/** The Tab component to hold all tabs. */
    private PnlTabs pnlTabs;

    /** The parent Frame that owns this JDialog. */
    private Frame parentFrame;

	/** The controller of the simulator. */
    private Controller controller;



    /* ===================================================================== */
    /** Constructor that creates this JDialog. The parent and controller must
     *  not be null for this Dialog to function properly.
     *
     *  @param  parent   The owning frame of this dialog.
     *  @param  controller   The simulation controller.
     *
     */
    public TrayDialog(Controller controller, Frame parent ) {

		super(parent);
        setTitle("Tray");

        this.parentFrame = parent;
        this.controller = controller;

		// Setup the gui
        try {
          init();
        }
        catch(Exception e) {
          e.printStackTrace();
        }


		// Set the size and location
        this.pack();
        this.setSize(470, 230);

        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (screen.width - this.getWidth()) - 50;
        int y = 50;

        this.setLocation(x,y);

		// Show us.
        this.setVisible(true);

    } //Construtor




    // =======================================================================
    /** Initializes the GUI with components.
     *
     */
    private void init() throws Exception {

		// Adjust the border layout
        this.getContentPane().setLayout(bdrLayMain);
        bdrLayMain.setHgap(2);
        bdrLayMain.setVgap(2);


		// Make our size fixed.
        this.setResizable(false);

		// Create a set of tabs
        pnlTabs = new PnlTabs(controller);

		// Add the tabs to this dialog.
        this.getContentPane().add(pnlTabs, BorderLayout.CENTER);


    } //method: init


} //class: TrayDialog
