/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package map;

import util.*;

/* ====================================================================== */
/** This class
 *
 *  A furnishing is always a rectangular object for now.
 *  1 foot = .000003 degrees of latitude/longitude
 */
public class Furnishing implements java.io.Serializable{


	/** The gps Position of top-left. */
  	private Position position1;

	/** The gps Position of bottom-right. */
  	private Position position2;

	/** The wireless signal degredation caused by this furnishing. */
  	private double rf_degredation;

	/** The sound level degredation caused by this furnishing. */
  	private double sound_degredation;

	/** The transparency of the furnishing. */
  	private double transparency;

  	/** The height of the furnishing (assume always sits on floor). */
  	private int height;

	/** Filename of image texture. */
	private String texture_filename;


    /* ===================================================================== */
    /** This constructor initializes the
     *
     *  @param  p1   The first endpoint.
     *  @param  p2   The second endpoint.
     *  @param  rf   The rf degredation.
     *  @param  sd   The sound degredation.
     *  @param  h    The height.
     *  @param  f    The wall texture.
     *  @param  t    The transparency.
     *
     */
  	public Furnishing(Position p1, Position p2, double rf, double sd, int h, String f, double t){

		position1 = p1;
		position2 = p2;
		rf_degredation = rf;
		sound_degredation = sd;
		height = h;
		texture_filename = f;
		transparency = t;

	} //Constructor





    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public Position getPosition1(){

		return position1;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public Position getPosition2(){

		return position2;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @param
     *
     */
	public void setPosition1(Position p){

		position1 = p;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @param
     *
     */
	public void setPosition2(Position p){

		position2 = p;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public double getRFDegredation(){

		return rf_degredation;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public double getSoundDegredation(){

		return sound_degredation;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setRFDegredation(double d){

		rf_degredation = d;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setSoundDegredation(double d){

		sound_degredation = d;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setTransparency(double d){

		transparency = d;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public int getHeight(){

		return height;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public String getTexture(){

		return texture_filename;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public double getTransparency(){

		return transparency;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setHeight(int d){

		height = d;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setTexture(String d){

		texture_filename = d;

	} //method:

} //class: Furnishing

